/*******************************************************************************
 *  emeter-template.h - MSP430F6736 distribution version
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

/* Define the following to build the software such that the info memory is overwritten
   each time new code is loaded into the MCU. */
#define OVERWRITE_INFOMEM

/*! This switch enables multi-rate tariff features */
#undef MULTI_RATE_SUPPORT

/*! This switch, in combination with the calibrator switch, enables calibration
    with the meter cooperating with an external reference, through a UART port. */
#define SERIAL_CALIBRATION_SUPPORT
#define SERIAL_CALIBRATION_PASSWORD_1               0x1234
#define SERIAL_CALIBRATION_PASSWORD_2               0x5678
#define SERIAL_CALIBRATION_PASSWORD_3               0x9ABC
#define SERIAL_CALIBRATION_PASSWORD_4               0xDEF0

/*! This switch enables the sending of the current readings, through a UART port,
    for use in cooperative calibration with other meters. */
#define SERIAL_CALIBRATION_REF_SUPPORT

/*! This selects real time clock support. This is implemented in software on
    the MSP430. */
#undef RTC_SUPPORT

/*! Related definitions to get special action routines to be called at various
    intervals. User supplied callback functions must be provided. Note these
    callback routines are called from within the per second timer interrupt
    service routine. Don't do anything too complex within them. If a long
    activity is required, set a flag within a simple routine, and do the main
    work in the main non-interrupt loop. */
#undef PER_SECOND_ACTIVITY_SUPPORT
#undef PER_MINUTE_ACTIVITY_SUPPORT
#undef PER_HOUR_ACTIVITY_SUPPORT
#undef PER_DAY_ACTIVITY_SUPPORT
#undef PER_MONTH_ACTIVITY_SUPPORT
#undef PER_YEAR_ACTIVITY_SUPPORT

/*! Corrected RTC support enables temperature and basic error compensation for
    the MSP430's 32kHz crystal oscillator, so it makes for a higher quality RTC
    source, even using low accuracy (eg 20ppm) crystals. */
#undef CORRECTED_RTC_SUPPORT

/* This switch enables power down to battery backup status on loss of line
   power. */
#undef POWER_DOWN_SUPPORT

/* These switches select a method of detecting when power is restored, if
   power down mode is supported. */
/* This method assumes the voltage waveform is being turned into simple
   digital pulses into an I/O pin. If this feature is used, POWER_UP_VOLTAGE_PULSE
   must define the way a voltage pulse is sensed. */
#undef POWER_UP_BY_VOLTAGE_PULSES

/* This method assumes the pre-regulator power supply voltage is being
   monitored by Comparator A. This method is suitable for meters which
   support a live/neutral only mode, for tamper resistance. */
#undef POWER_UP_BY_SUPPLY_SENSING

/* Many data logging requirements can be met by using only the MSP430's
   info memory. If an external serial EEPROM is needed for more
   complex requirements, this switch will enable an interface to
   I2C type serial EEPROMs. */
#undef EXTERNAL_EEPROM_SUPPORT
#undef EEPROM_IS_REALLY_FRAM
#define EEPROM_IS_24C02
#define USCIB0_IS_IIC_EEPROM
#define EEPROM_PORT_SEL                             P2SEL
#define EEPROM_PORT_OUT                             P2OUT
#define EEPROM_PORT_DIR                             P2DIR
#define EEPROM_PORT_REN                             P2REN
#define EEPROM_SDA_PIN                              BIT1
#define EEPROM_SCL_PIN                              BIT0

/*! This defines the speed of USART 0 or USCI 0 */
#define UART_0_SUPPORT
#define UART_0_BAUD_RATE                            9600
#define UART_0_DLT645_SUPPORT

/*! This defines the speed of USART 1 or USCI 1 */
#undef UART_1_SUPPORT
#undef UART_1_BAUD_RATE                            9600
#undef UART_1_DLT645_SUPPORT

#undef INFRA_RED_38K_SUPPORT
#undef INFRA_RED_38K_FROM_TIMER_A
#undef INFRA_RED_38K_FROM_TIMER_B

/* Pull in definitions for the LCD */
#include "ti_lcd.h"

/*! This switch enables multi-rate tariff features */
#undef MULTI_RATE_SUPPORT

/*! This switch selects where a backup battery's condition is monitored. */
#undef BATTERY_MONITOR_SUPPORT

/*! This definition specifies a function to handle the tamper indications

    The limp indicator will be set when the meter is in limp mode.

    The earthed indicator will be set when there is a substantial imbalance in
    power (normal operating mode) or current (limp mode) between the live and
    neutral leads. If the neutral has the higher reading, this will also be used
    instead of the live reading.

    The reversed indicator will be set in normal operating mode, if the raw
    power reading is negative in the live lead (without neutral lead monitoring)
    or in either of the leads (with neutral lead monitoring). The power value
    used is simply minus the actual reading. This only functions in the normal
    operating mode. In limp mode we have no simple way to tell forward current
    flow from reverse. In limp mode the LED is switched off. Note that if the
    meter is used in applications where negative power means we are actively
    feeding power to the public grid this functionality is inappropriate. In
    those cases the negative power would generally be used to accumulate a
    separate "generated energy" reading, to complement the usual
    "consumed energy" reading. */
#define tamper_event(type,start) \
{ \
    switch (type) \
    { \
    case TAMPER_TYPE_REVERSAL: \
        if (start) \
            /* Turn on the current reversed indicator */; \
        else \
            /* Turn off the current reversed indicator */; \
        break; \
    case TAMPER_TYPE_BYPASS: \
        if (start) \
            /* Turn on the current bypassed/earthed indicator */; \
        else \
            /* Turn off the current bypassed/earthed indicator */; \
        break; \
    case TAMPER_TYPE_LIMP: \
        if (start) \
            /* Turn on the limp mode indicator */; \
        else \
            /* Turn off the limp mode indicator */; \
        break; \
    } \
}

#define custom_set_consumption(x,y)                 /**/

#undef LCD_DISPLAY_SUPPORT

#define USE_STARBURST
#define USE_7SEGMENT

#define STARBURST_FIELD                             1
#define NUMERIC_FIELD                               2

#define INFO_POSITION                               STARBURST_FIRST_DIGIT
#define INFO_CHARS                                  STARBURST_DIGITS

#undef LCD_BARGRAPH_SUPPORT

#define TIME_DISPLAY_FIELD                          STARBURST_FIELD
#define TIME_TAG                                    "Time"
#define DATE_DISPLAY_FIELD                          STARBURST_FIELD
#define DATE_TAG                                    "Date"

#define HOUR_DISPLAY_POSITION                       1
#define ICON_TIME_COLON_1                           LCD_STARBURST_COLON1
#define MINUTE_DISPLAY_POSITION                     3
#define ICON_TIME_COLON_2                           LCD_STARBURST_COLON2
#define SECONDS_DISPLAY_POSITION                    5

#define YEAR_DISPLAY_POSITION                       1
#define ICON_DATE_COLON_1                           LCD_STARBURST_COLON1
#define MONTH_DISPLAY_POSITION                      3
#define ICON_DATE_COLON_2                           LCD_STARBURST_COLON2
#define DAY_DISPLAY_POSITION                        5

/* We have no icons for V, kW, etc, so we need to display a character to show the information type
   on display */
#define DISPLAY_TYPE_FIELD                          2
#define DISPLAY_TYPE_POSITION                       1

#define ACTIVE_POWER_DISPLAY_FIELD                  STARBURST_FIELD
#define ACTIVE_POWER_TAG                            "AcPo"

#define REACTIVE_POWER_DISPLAY_FIELD                STARBURST_FIELD
#define REACTIVE_POWER_TAG                          "rePo"

#define APPARENT_POWER_DISPLAY_FIELD                STARBURST_FIELD
#define APPARENT_POWER_TAG                          "ApPo"

#define ACTIVE_ENERGY_DISPLAY_FIELD                 STARBURST_FIELD
#define ACTIVE_ENERGY_DISPLAY_IN_KWH
#define ACTIVE_ENERGY_TAG                           "AcEn"

#define REACTIVE_ENERGY_DISPLAY_FIELD               STARBURST_FIELD
#define REACTIVE_ENERGY_DISPLAY_IN_KVARH
#define REACTIVE_ENERGY_TAG                         "reEn"

#define POWER_FACTOR_DISPLAY_FIELD                  STARBURST_FIELD
#define POWER_FACTOR_TAG                            "PF"

#define VOLTAGE_DISPLAY_FIELD                       STARBURST_FIELD
#define VOLTAGE_TAG                                 "Vrms"

#define CURRENT_DISPLAY_FIELD                       STARBURST_FIELD
#define CURRENT_TAG                                 "Irms"

#define FREQUENCY_DISPLAY_FIELD                     STARBURST_FIELD
#define FREQUENCY_TAG                               "Freq"

#define TEMPERATURE_DISPLAY_FIELD                   STARBURST_FIELD
#define TEMPERATURE_TAG                             "Temp"

/* Definition of the minus sign segments and decimal point segments for the display fields */
#define DISPLAY_MINUS_SIGNS \
 \
static const uint8_t field1_minus_icons[] = \
{ \
    LCD_STARBURST_MINUS_1, \
    LCD_STARBURST_MINUS_2, \
    LCD_STARBURST_MINUS_3, \
    LCD_STARBURST_MINUS_4, \
    LCD_STARBURST_MINUS_5, \
    LCD_STARBURST_MINUS_6 \
}; \
 \
static const uint8_t field2_minus_icons[] = \
{ \
    LCD_7SEGMENT_MINUS_1, \
    LCD_7SEGMENT_MINUS_2, \
    LCD_7SEGMENT_MINUS_3, \
    LCD_7SEGMENT_MINUS_4 \
};

#define DISPLAY_DECIMAL_POINTS \
static const uint8_t field1_dp_icons[] = \
{ \
    0, \
    LCD_STARBURST_DP_5, \
    LCD_STARBURST_DP_4, \
    LCD_STARBURST_DP_3, \
    LCD_STARBURST_DP_2, \
    LCD_STARBURST_DP_1 \
}; \
 \
static const uint8_t field2_dp_icons[] = \
{ \
    0, \
    LCD_7SEGMENT_DP_3, \
    LCD_7SEGMENT_DP_2, \
    LCD_7SEGMENT_DP_1 \
};

/* LCD display fields
   The main field is a 6 character starburst field.
   The supplementary field is a 4 character 7-segment field */
#define DISPLAY_FIELDS \
    {1, 2, 6, LCD_MODE_STARBURST, LCD_STARBURST_MASK, field1_minus_icons, field1_dp_icons}, \
    {17, -1, 4, LCD_MODE_7SEGMENT, LCD_7SEGMENT_MASK, field2_minus_icons, field2_dp_icons}

#define NUM_DISPLAY_FIELDS 2

/* LCD display sequence table */
#define DISPLAY_STEP_SEQUENCE \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_ACTIVE_POWER, \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_REACTIVE_POWER, \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_VOLTAGE, \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_CURRENT, \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_MAINS_FREQUENCY, \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_ACTIVE_ENERGY, \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_REACTIVE_ENERGY, \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_TIME, \
    DISPLAY_ITEM_SELECT_PHASE_1, \
        DISPLAY_ITEM_DATE, \
    DISPLAY_ITEM_SELECT_RESTART

#define custom_lcd_clear_periphery() \
    lcd_cell(0, BARGRAPH_DIGIT - 1); \
    lcd_cell(0, PIE_CHART_DIGIT - 1); \
    lcd_icon(LCD_7SEGMENT_PM, false); \
    lcd_icon(LCD_7SEGMENT_BELL, false); \
    lcd_icon(LCD_7SEGMENT_MINUS_1, false); \
    lcd_icon(LCD_7SEGMENT_COLON, false); \
    lcd_icon(LCD_7SEGMENT_DP_1, false); \
    lcd_icon(LCD_7SEGMENT_DP_2, false); \
    lcd_icon(LCD_7SEGMENT_DP_3, false); \
    lcd_icon(LCD_STARBURST_MINUS_1, false); \
    lcd_icon(LCD_STARBURST_COLON1, false); \
    lcd_icon(LCD_STARBURST_COLON2, false); \
    lcd_icon(LCD_STARBURST_DEGREE, false); \
    lcd_icon(LCD_STARBURST_DP_1, false); \
    lcd_icon(LCD_STARBURST_DP_2, false); \
    lcd_icon(LCD_STARBURST_DP_3, false); \
    lcd_icon(LCD_STARBURST_DP_4, false); \
    lcd_icon(LCD_STARBURST_DP_5, false); \
    lcd_icon(LCD_COMMA, false); \
    lcd_icon(LCD_RING, false); \
    lcd_icon(LCD_HEART, false); \
    lcd_icon(LCD_EXCLAMATION_MARK, false); \
    lcd_icon(LCD_MAST, false); \
    lcd_icon(LCD_MAST_RX, false); \
    lcd_icon(LCD_MAST_TX, false); \
    lcd_icon(LCD_TX_SMALL_RIGHT, false); \
    lcd_icon(LCD_TX_LARGE_RIGHT, false); \
    lcd_icon(LCD_TX_SMALL_LEFT, false); \
    lcd_icon(LCD_TX_LARGE_LEFT, false); \
    lcd_icon(LCD_BOXED_R, false); \
    lcd_icon(LCD_PIE_CHART_PIP_1, false); \
    lcd_icon(LCD_PIE_CHART_PIP_2, false); \
    lcd_icon(LCD_PIE_CHART_PIP_3, false); \
    lcd_icon(LCD_PIE_CHART_PIP_4, false);

#define custom_lcd_clear_line_1_tags()
#define custom_lcd_clear_line_2_tags()

#define custom_energy_pulse_start(ph, which) \
{ \
    switch (which) \
    { \
    case ACTIVE_ENERGY_IMPORTED: \
        P1OUT &= ~BIT6; \
        P8OUT &= ~BIT6; \
        break; \
    case REACTIVE_ENERGY_QUADRANT_I: \
        P1OUT &= ~BIT7; \
        P8OUT &= ~BIT7; \
        break; \
    } \
}

#define custom_energy_pulse_end(ph, which) \
{ \
    switch (which) \
    { \
    case ACTIVE_ENERGY_IMPORTED: \
        P1OUT |= BIT6; \
        P8OUT |= BIT6; \
        break; \
    case REACTIVE_ENERGY_QUADRANT_I: \
        P1OUT |= BIT7; \
        P8OUT |= BIT7; \
        break; \
    } \
}

/* This custom function must check the conditions, such as passswords and shorting link tests,
   appropriate to the meter's "enable calibration" condition. */
#define custom_is_calibration_enabled() true

#undef USE_WATCHDOG

/*
    P1.0 =
    P1.1 =
    P1.2 = UART 0 Rx
    P1.3 = UART 0 Tx
    P1.4 = UART 1 Rx
    P1.5 = UART 1 Tx
    P1.6 = GFCI Fault
    P1.7 = GFCI Test
 */
#define P1DIR_INIT                                  (BIT7 | BIT5 | BIT3)
#define P1SEL_INIT                                  (BIT5 | BIT4 | BIT3 | BIT2)
#define P1OUT_INIT                                  0

/*
    P2.0 = SCL for EEPROM (USCIB0)
    P2.1 = SDA for EEPROM (USCIB0)
    P2.2 =
    P2.3 =
    P2.4 = 
    P2.5 = 
    P2.6 = Button 1 
    P2.7 = Button 2
 */
#define P2DIR_INIT                                  (0)
#define P2SEL_INIT                                  (BIT1 | BIT0)
#define P2OUT_INIT                                  (0)

/*
    P3.0 =
    P3.1 = Voltage Check
    P3.2 = RF GPIO2
    P3.3 = RF GPIO1
    P3.4 = LED1
    P3.5 = LED2
    P3.6 =
    P3.7 =
 */
#define P3DIR_INIT                                  (BIT4 | BIT5)
#define P3SEL_INIT                                  (0)
#define P3OUT_INIT                                  (BIT4 | BIT5)

/*
    P4.0 = Relay 1
    P4.1 = Relay 2
    P4.2 = Relay 3
    P4.3 = Relay 4
    P4.4 =
    P4.5 =
    P4.6 =
    P4.7 =
 */
#define P4DIR_INIT                                  (BIT0 | BIT1 | BIT2 | BIT3)
#define P4SEL_INIT                                  0
#define P4OUT_INIT                                  0

/*
    P5.0 =
    P5.1 =
    P5.2 =
    P5.3 =
    P5.4 =
    P5.5 =
    P5.6 =
    P5.7 =
*/
#define P5DIR_INIT                                  0
#define P5SEL_INIT                                  0
#define P5OUT_INIT                                  0

/*
    P6.0 =
    P6.1 =
    P6.2 =
    P6.3 =
    P6.4 =
    P6.5 =
    P6.6 =
    P6.7 =
*/
#define P5DIR_INIT                                  0
#define P5SEL_INIT                                  0
#define P5OUT_INIT                                  0

/*
    P7.0 =
    P7.1 =
    P7.2 =
    P7.3 =
    P7.4 =
    P7.5 =
    P7.6 =
    P7.7 =
 */
#define P7DIR_INIT                                  0
#define P7SEL_INIT                                  0
#define P7OUT_INIT                                  0

/*
    P8.0 =
    P8.1 =
    P8.2 =
    P8.3 =
    P8.4 = RF VREG EN
    P8.5 = RF Reset CC
    P8.6 = RF CCA
    P8.7 = TA2.0 - Pilot Out - Default Low
 */
#define P8DIR_INIT                                  (BIT7)
#define P8SEL_INIT                                  0
#define P8OUT_INIT                                  0
#define P8REN_INIT                                  (BIT7)

/*
    P9.0 = RF SFD
    P9.1 =
    P9.2 =
    P9.3 = A3 - PILOT IN
    P9.4 = 
    P9.5 = 
    P9.6 = 
    P9.7 = 
 */
#define P9DIR_INIT                                  (0)
#define P9SEL_INIT                                  (BIT3)
#define P9OUT_INIT                                  (0)

/*
    PJ.0 = SMCLK
    PJ.1 = MCLK
    PJ.2 = ADC10CLK
    PJ.3 = ACLK
 */
#define PJDIR_INIT                                  (BIT3 | BIT1 | BIT0)
#define PJSEL_INIT                                  (BIT3 | BIT1 | BIT0)
#define PJOUT_INIT                                  (0)

/*
    The LCD is a full 160 segment display, with 1 line of 6 starburst digits,
    1 line of 4 7-seg digits, plus various symbols.
 */
#define LCDCCTL0_INIT                               (LCDDIV_31 | LCDPRE_1 | LCD4MUX | LCDON)
#define LCDCCTL1_INIT                               (0)
#define LCDCBLKCTL_INIT                             (0)
#define LCDCMEMCTL_INIT                             (0)
#define LCDCVCTL_INIT                               (LCDCPEN | VLCD_2_60)
#define LCDCPCTL0_INIT                              (0xFFFF)  /* Segment lines 0 to 39 */
#define LCDCPCTL1_INIT                              (0xFFFF)
#define LCDCPCTL2_INIT                              (0x00FF)
#define LCDCCPCTL_INIT                              (0)

#undef SUPPORT_ONE_POINT_AUTOCALIBRATION
#define AUTOCAL_VOLTAGE                     220
#define AUTOCAL_CURRENT                     5

/* We assume that the phase shift between voltage and current is exactly 60 degrees, and the frequency is
   exactly the nominal mains frequency. */

/* The following is the maximum percentage error expected in the readings before calibration.
   It is used as a sanity check during autocalibration, so we don't try to calibrate on a
   crazy signal level. */
#define AUTOCAL_MAX_UNCAL_ERROR             10
/* Button 4 selects autocalibration */
#define AUTOCAL_ENABLE_PORT                 P1IN
#define AUTOCAL_ENABLE_BIT                  BIT0
#define AUTOCAL_ENABLE_BIT_ACTIVE_STATE     0

#define custom_autocal_indicator(state) \
{ \
    switch (state) \
    { \
    case 0: \
        /* Auto-calibration not in progress */ \
        P5OUT |= (BIT0 | BIT1); \
        break; \
    case 1: \
        /* Auto-calibration not in progress, but signal not stable */ \
        P5OUT &= ~BIT0; \
        P5OUT |= BIT1; \
        break; \
    case 2: \
        /* Auto-calibration not in progress, and gathering stable signals */ \
        P5OUT |= BIT0; \
        P5OUT &= ~BIT1; \
        break; \
    case 3: \
        /* Auto-calibration in the winding down phase */ \
        P5OUT &= ~(BIT0 | BIT1); \
        break; \
    } \
}
