/*******************************************************************************
 *  emeter-iic-eeprom.h -
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

/*! \file */

#if !defined(_EMETER_IIC_EEPROM_H_)
#define _EMETER_IIC_EEPROM_H_

#if defined(EEPROM_IS_24C01)
#define EEPROM_ADDRESS_BITS                 7
#define EEPROM_ADDRESS_BYTES                1
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     0
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    3
#define EEPROM_PAGE_SIZE                    8
#elif defined(EEPROM_IS_24C02)
#define EEPROM_ADDRESS_BITS                 8
#define EEPROM_ADDRESS_BYTES                1
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     0
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    3
#define EEPROM_PAGE_SIZE                    8
#elif defined(EEPROM_IS_24C04)
#define EEPROM_ADDRESS_BITS                 9
#define EEPROM_ADDRESS_BYTES                1
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     1
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    2
#define EEPROM_PAGE_SIZE                    16
#elif defined(EEPROM_IS_24C08)
#define EEPROM_ADDRESS_BITS                 10
#define EEPROM_ADDRESS_BYTES                1
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     2
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    1
#define EEPROM_PAGE_SIZE                    16
#elif defined(EEPROM_IS_24C16)
#define EEPROM_ADDRESS_BITS                 11
#define EEPROM_ADDRESS_BYTES                1
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     3
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    0
#define EEPROM_PAGE_SIZE                    16
#elif defined(EEPROM_IS_24C32)
#define EEPROM_ADDRESS_BITS                 12
#define EEPROM_ADDRESS_BYTES                1
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     0
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    3
#define EEPROM_PAGE_SIZE                    32
#elif defined(EEPROM_IS_24C64)
#define EEPROM_ADDRESS_BITS                 13
#define EEPROM_ADDRESS_BYTES                2
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     0
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    3
#define EEPROM_PAGE_SIZE                    32
#elif defined(EEPROM_IS_24C128)
#define EEPROM_ADDRESS_BITS                 14
#define EEPROM_ADDRESS_BYTES                2
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     0
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    2
#define EEPROM_PAGE_SIZE                    64
#elif defined(EEPROM_IS_24C256)
#define EEPROM_ADDRESS_BITS                 15
#define EEPROM_ADDRESS_BYTES                2
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     0
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    2
#define EEPROM_PAGE_SIZE                    64
#elif defined(EEPROM_IS_24C512)
#define EEPROM_ADDRESS_BITS                 16
#define EEPROM_ADDRESS_BYTES                2
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     0
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    2
#define EEPROM_PAGE_SIZE                    128
#elif defined(EEPROM_IS_24C1024)
#define EEPROM_ADDRESS_BITS                 17
#define EEPROM_ADDRESS_BYTES                2
#define EEPROM_ADDRESS_BITS_IN_I2C_ADDR     1
#define EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR    1
#define EEPROM_PAGE_SIZE                    256
#endif

#define EEPROM_SLAVE_ADDRESS                0x50

enum
{
    EEPROM_RESULT_IN_PROGRESS = 1,
    EEPROM_RESULT_OK = 0,
    EEPROM_RESULT_NACK = -1,
    EEPROM_RESULT_CLOCK_STUCK = -2,
    EEPROM_RESULT_AL = -3,
    EEPROM_RESULT_TIMEOUT = -4
};

typedef uint32_t eeprom_addr_t;

int external_eeprom_init(void);

int eeprom_write_byte(eeprom_addr_t address, uint8_t data);
int eeprom_write(eeprom_addr_t address, const uint8_t buf[], unsigned int size);

int eeprom_read_byte(eeprom_addr_t sddress);
int eeprom_read_next_byte(void);
int eeprom_read(eeprom_addr_t address, uint8_t buf[], unsigned int size);

/* Look for an ACK from the EEPROM, with a timeout. This is usually used to determine
   when the slow internal write operation of the EEPROM has completed. If the device
   is actually an FRAM, this kind of check is not required. */
int eeprom_wait_for_ack(void);

void i2c_eeprom_rx_core(int port);

#endif
