/*******************************************************************************
 *  emeter-usci-iic-eeprom.c -
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

#include <inttypes.h>
#include <stdbool.h>
#if defined(__MSP430__)
#include <msp430.h>
#endif

#include <emeter-toolkit.h>

#include "emeter-template.h"
#include "emeter-iic-eeprom.h"

#if     defined(__MSP430_HAS_EUSCI_B0__) \
    ||  defined(__MSP430_HAS_EUSCI_B1__) \
    ||  defined(__MSP430_HAS_EUSCI_B2__) \
    ||  defined(__MSP430_HAS_EUSCI_B3__)
#define __MSP430_HAS_EUSCI__
#elif   defined(__MSP430_HAS_USCI_AB0__) \
    ||  defined(__MSP430_HAS_USCI_AB1__)
#define __MSP430_HAS_USCI_AB__
#endif

/* Allow for any of the possible I2C ports being the one used for the EEPROM devices */
#if defined(USCIB0_IS_IIC_EEPROM)
#define UCBxCTL0        UCB0CTL0
#define UCBxCTL1        UCB0CTL1
#define UCBxCTLW0       UCB0CTLW0
#define UCBxCTLW1       UCB0CTLW1
#define UCBxBR0         UCB0BR0
#define UCBxBR1         UCB0BR1
#define UCBxI2CIE       UCB0I2CIE
#define UCBxSTAT        UCB0STAT
#define UCBxRXBUF       UCB0RXBUF
#define UCBxTXBUF       UCB0TXBUF
#define UCBxI2COA       UCB0I2COA
#define UCBxI2CSA       UCB0I2CSA
#define UCBxTBCNT       UCB0TBCNT
#define UCBxBCNT        UCB0BCNT
#if defined(__MSP430_HAS_USCI_AB__)
#define UCxIE           IE2
#define UCxIFG          IFG2
#else
#define UCBxIE          UCB0IE
#define UCBxIFG         UCB0IFG
#define UCBxIV          UCB0IV
#endif

#define UCBxRXIFG       UCB0RXIFG
#define UCBxTXIFG       UCB0TXIFG
#define UCBxRXIE        UCB0RXIE
#define UCBxTXIE        UCB0TXIE
#elif defined(USCIB1_IS_IIC_EEPROM)
#define UCBxCTL0        UCB1CTL0
#define UCBxCTL1        UCB1CTL1
#define UCBxCTLW0       UCB1CTLW0
#define UCBxCTLW1       UCB1CTLW1
#define UCBxBR0         UCB1BR0
#define UCBxBR1         UCB1BR1
#define UCBxI2CIE       UCB1I2CIE
#define UCBxSTAT        UCB1STAT
#define UCBxRXBUF       UCB1RXBUF
#define UCBxTXBUF       UCB1TXBUF
#define UCBxI2COA       UCB1I2COA
#define UCBxI2CSA       UCB1I2CSA
#define UCBxTBCNT       UCB1TBCNT
#define UCBxBCNT        UCB1BCNT
#if defined(__MSP430_HAS_USCI_AB__)
#define UCxIE           UC1IE
#define UCxIFG          UC1IFG
#else
#define UCBxIE          UCB1IE
#define UCBxIFG         UCB1IFG
#define UCBxIV          UCB1IV
#endif

#define UCBxRXIFG       UCB1RXIFG
#define UCBxTXIFG       UCB1TXIFG
#define UCBxRXIE        UCB1RXIE
#define UCBxTXIE        UCB1TXIE
#elif defined(USCIB2_IS_IIC_EEPROM)
#define UCBxCTL0        UCB2CTL0
#define UCBxCTL1        UCB2CTL1
#define UCBxCTLW0       UCB2CTLW0
#define UCBxCTLW1       UCB2CTLW1
#define UCBxBR0         UCB2BR0
#define UCBxBR1         UCB2BR1
#define UCBxI2CIE       UCB2I2CIE
#define UCBxSTAT        UCB2STAT
#define UCBxRXBUF       UCB2RXBUF
#define UCBxTXBUF       UCB2TXBUF
#define UCBxI2COA       UCB2I2COA
#define UCBxI2COA0      UCB2I2COA0
#define UCBxI2CSA       UCB2I2CSA
#define UCBxTBCNT       UCB2TBCNT
#define UCBxBCNT        UCB2BCNT
#if defined(__MSP430_HAS_USCI_AB__)
#define UCxIE           UC2IE
#define UCxIFG          UC2IFG
#else
#define UCBxIE          UCB2IE
#define UCBxIFG         UCB2IFG
#define UCBxIV          UCB2IV
#endif

#define UCBxRXIFG       UCB2RXIFG
#define UCBxTXIFG       UCB2TXIFG
#define UCBxRXIE        UCB2RXIE
#define UCBxTXIE        UCB2TXIE
#elif defined(USCIB3_IS_IIC_EEPROM)
#define UCBxCTL0        UCB3CTL0
#define UCBxCTL1        UCB3CTL1
#define UCBxCTLW0       UCB3CTLW0
#define UCBxCTLW1       UCB3CTLW1
#define UCBxBR0         UCB3BR0
#define UCBxBR1         UCB3BR1
#define UCBxI2CIE       UCB3I2CIE
#define UCBxSTAT        UCB3STAT
#define UCBxRXBUF       UCB3RXBUF
#define UCBxTXBUF       UCB3TXBUF
#define UCBxI2COA       UCB3I2COA
#define UCBxI2CSA       UCB3I2CSA
#define UCBxTBCNT       UCB3TBCNT
#define UCBxBCNT        UCB3BCNT
#if defined(__MSP430_HAS_USCI_AB__)
#define UCxIE           UC3IE
#define UCxIFG          UC3IFG
#else
#define UCBxIE          UCB3IE
#define UCBxIFG         UCB3IFG
#define UCBxIV          UCB3IV
#endif

#define UCBxRXIFG       UCB3RXIFG
#define UCBxTXIFG       UCB3TXIFG
#define UCBxRXIE        UCB3RXIE
#define UCBxTXIE        UCB3TXIE
#endif

static uint8_t rx_count;
static uint8_t rx_len;
static uint8_t *rx_buf;

static uint8_t tx_count;
#if !defined(__MSP430_HAS_EUSCI__)
static uint8_t tx_len;
#endif
static uint8_t *tx_buf;
static uint8_t addr_buf[EEPROM_ADDRESS_BYTES];

static volatile int result;

int external_eeprom_init(void)
{
    EEPROM_PORT_OUT |= (EEPROM_SDA_PIN | EEPROM_SCL_PIN);
#if defined(EEPROM_PORT_REN)
    EEPROM_PORT_REN |= (EEPROM_SDA_PIN | EEPROM_SCL_PIN);
#endif
    EEPROM_PORT_SEL |= (EEPROM_SDA_PIN | EEPROM_SCL_PIN);

#if defined(__MSP430_HAS_EUSCI__)
    /* Most things in the module can only be changes in the SW reset state */
    UCBxCTLW0 = UCSWRST;
    /* I2C, master mode, synchronous mode, use SMCLK */
    UCBxCTLW0 = UCMST | UCMODE_3 | UCSYNC | UCSSEL_2 | UCSWRST;
    /* Auto stop if TBCNT is reached */
    UCBxCTLW1 = UCASTP_2;
#else
    /* I2C, master mode, synchronous mode */
    UCBxCTL0 = UCMST | UCMODE_3 | UCSYNC;
    /* Use SMCLK, keep SW reset */
    UCBxCTL1 = UCSSEL_2 | UCSWRST;
#endif
    /* Set the I2C clock to 100kbps */
    #if defined(__MSP430F6736__)  ||  defined(__MSP430F6749__)  ||  defined(__MSP430F67491__)  ||  defined(__MSP430F6779__)  ||  defined(__MSP430F67791__)
    /* 24MHz clock */
    UCB0BR0 = 32768L*768L/100000L;
    #elif defined(__MSP430F4794__)  ||  defined(__MSP430F47197__)
    /* 16MHz clock */
    UCB0BR0 = 32768L*512L/100000L;
    #elif defined(__MSP430AFE251__)  ||  defined(__MSP430AFE252__)  ||  defined(__MSP430AFE253__)
    /* Exactly 8MHz clock */
    UCB0BR0 = 8000000L/100000L;
    #else
    /* 8MHz clock */
    UCB0BR0 = 32768L*256L/100000L;
    #endif
    UCBxBR1 = 0;
    /* Set an initial slave address for the EEPROM, although this might be changed
       dynamically if the slace address includes a part of the memory address. */
    UCBxI2CSA = EEPROM_SLAVE_ADDRESS;
#if defined(__MSP430_HAS_USCI_AB__)
    UCxIE &= ~(UCBxRXIE | UCBxTXIE);
#else
    UCBxIE = 0;
#endif
    /* Let the module run */
    UCBxCTL1 &= ~UCSWRST;

    if ((UCBxSTAT & UCBBUSY))
    {
        /* We need to control the clock manually, to clear the bus */
        EEPROM_PORT_SEL &= ~EEPROM_SCL_PIN;
        EEPROM_PORT_OUT &= ~EEPROM_SCL_PIN;
        EEPROM_PORT_DIR |= EEPROM_SCL_PIN;
        __delay_cycles(100);
        EEPROM_PORT_DIR &= ~EEPROM_SCL_PIN;
        EEPROM_PORT_SEL |= EEPROM_SCL_PIN;
    }

    /* Wait for the I2C bus to be free */
    while ((UCBxSTAT & UCBBUSY));
#if defined(__MSP430_HAS_USCI_AB__)
    UCxIE |= (UCBxRXIE | UCBxTXIE);
    UCBxI2CIE |= (UCSTPIE | UCSTTIE);
#elif defined(__MSP430_HAS_EUSCI__)
    UCBxIE = UCRXIE | UCTXIE | UCSTPIE | UCNACKIE | UCCLTOIE;
#else
    UCBxIE = UCRXIE | UCTXIE | UCSTPIE | UCNACKIE;
#endif
    return 0;
}

static unsigned int set_address(eeprom_addr_t address)
{
#if EEPROM_ADDRESS_BITS_IN_I2C_ADDR > 0  ||  EEPROM_CHIP_SEL_BITS_IN_I2C_ADDR > 0
    UCBxI2CSA = EEPROM_SLAVE_ADDRESS | (address >> (EEPROM_ADDRESS_BITS - EEPROM_ADDRESS_BITS_IN_I2C_ADDR));
#else
    UCBxI2CSA = EEPROM_SLAVE_ADDRESS;
#endif
#if EEPROM_ADDRESS_BYTES == 1
    addr_buf[0] = address & 0xFF;
#else
    addr_buf[0] = address >> 8;
    addr_buf[1] = address & 0xFF;
#endif
    return EEPROM_ADDRESS_BYTES;
}

static void i2c_write_init(void)
{
    result = EEPROM_RESULT_IN_PROGRESS;
    tx_count = 0;
#if defined(__MSP430_HAS_USCI_AB__)
    /* Clear the TX ready and stop interrupt flags */
    UCxIFG &= ~UCBxTXIFG;
#else
    /* Clear the TX ready and stop interrupt flags */
    UCBxIFG &= ~(UCTXIFG | UCSTPIFG);
#endif
    /* I2C transmit mode, start condition */
    UCBxCTL1 |= (UCTR | UCTXSTT);
}

static void i2c_read_init(void)
{
    /* No auto stop if TBCNT is reached */
    /* I2C receive mode */
    UCBxCTL1 &= ~UCTR;
}

int eeprom_write_byte(eeprom_addr_t address, uint8_t data)
{
    unsigned int len;

    /* Wait for the I2C bus to be free */
    while ((UCBxSTAT & UCBBUSY));

    rx_len = 0;
    len = set_address(address);
#if !defined(__MSP430_HAS_EUSCI__)
    tx_len = len + 1;
#else
    UCBxTBCNT = len + 1;
#endif
    tx_buf = &data - len;
    i2c_write_init();
    while (result == EEPROM_RESULT_IN_PROGRESS);
#if defined(EEPROM_IS_REALLY_FRAM)
    return result;
#else
    if (result != EEPROM_RESULT_OK)
        return result;
    /* Wait for the write to complete inside the EEPROM */
    return eeprom_wait_for_ack();
#endif
}

int eeprom_write(eeprom_addr_t address, const uint8_t buf[], unsigned int size)
{
    unsigned int ptr;
    unsigned int len_addr;
    unsigned int len_data;

    if (size == 0)
        return 0;

    /* Wait for the I2C bus to be free */
    while (UCBxSTAT & UCBBUSY);

    /* Write an arbitrarily sized buffer, in EEPROM page limited chunks */
    for (ptr = 0;  ptr < size;  ptr += len_data)
    {
        /* We need to limit the size of this write to remain within the current EEPROM page */
        len_data = EEPROM_PAGE_SIZE - (address & (EEPROM_PAGE_SIZE - 1));
        if (len_data > (size - ptr))
            len_data = size - ptr;

        rx_len = 0;
        len_addr = set_address(address);
#if !defined(__MSP430_HAS_EUSCI__)
        tx_len = len_addr + len_data;
#else
        UCBxTBCNT = len_addr + len_data;
#endif
        tx_buf = (uint8_t *) &buf[ptr] - len_addr;
        i2c_write_init();
        /* Wait for the I2C bus to be free */
        while (result == EEPROM_RESULT_IN_PROGRESS);
        if (result != EEPROM_RESULT_OK)
            return result;
#if !defined(EEPROM_IS_REALLY_FRAM)
        /* Wait for the write to complete inside the EEPROM */
        result = eeprom_wait_for_ack();
        if (result != EEPROM_RESULT_OK)
            return result;
#endif
        address += len_data;
    }
    return size;
}

int eeprom_read_next_byte(void)
{
    uint8_t data;

    /* Wait for the I2C bus to be free */
    while ((UCBxSTAT & UCBBUSY));

    result = EEPROM_RESULT_IN_PROGRESS;
    rx_buf = &data;
    rx_len = 1;
    rx_count = 0;
    i2c_read_init();
    /* I2C TX, start condition */
    UCBxCTL1 |= UCTXSTT;
    while (result == EEPROM_RESULT_IN_PROGRESS);
    if (result != EEPROM_RESULT_OK)
        return result;
    return data;
}

int eeprom_read_byte(eeprom_addr_t address)
{
    unsigned int len;
    uint8_t data;

    /* Wait for the I2C bus to be free */
    while ((UCBxSTAT & UCBBUSY));

    rx_buf = &data;
    rx_len = 1;
    rx_count = 0;
    len = set_address(address);
#if !defined(__MSP430_HAS_EUSCI__)
    tx_len = len;
#else
    UCBxTBCNT = len;
#endif
    i2c_write_init();
    while (result == EEPROM_RESULT_IN_PROGRESS);
    if (result != EEPROM_RESULT_OK)
        return result;
    return data;
}

int eeprom_read(eeprom_addr_t address, uint8_t *data, unsigned int size)
{
    unsigned int len;

    if (size == 0)
        return 0;

    /* Wait for the I2C bus to be free */
    while ((UCBxSTAT & UCBBUSY));

    rx_buf = data;
    rx_len = size;
    rx_count = 0;
    len = set_address(address);
#if !defined(__MSP430_HAS_EUSCI__)
    tx_len = len;
#else
    UCBxTBCNT = len;
#endif
    i2c_write_init();
    while (result == EEPROM_RESULT_IN_PROGRESS);
    if (result != EEPROM_RESULT_OK)
        return result;
    return rx_count;
}

int eeprom_wait_for_ack(void)
{
    uint8_t ucbxie_bak;
    unsigned int timeout;

#if defined(__MSP430_HAS_USCI_AB__)
    /* Save IE register */
    ucbxie_bak = UCxIE & (UCBxRXIE | UCBxTXIE);
    /* No RX or TX interrupts */
    UCxIE &= ~(UCBxRXIE | UCBxTXIE);
    /* No NACK interrupts */
    UCBxI2CIE &= ~UCNACKIE;
#else
    /* Save IE register */
    ucbxie_bak = UCBxIE;
    /* No interrupts */
    UCBxIE = 0;
#endif

    /* Wait for the I2C bus to be free */
    while ((UCBxSTAT & UCBBUSY));

    /* Repeat sending until its ACK'ed */
    for (timeout = 1000;  timeout > 0;  timeout--)
    {
        /* Clear the NACK flag */
#if defined(__MSP430_HAS_USCI_AB__)
        UCxIFG &= ~UCNACKIFG;
#else
        UCBxIFG &= ~UCNACKIFG;
#endif
        /* I2C TX, start condition, stop condition */
        UCBxCTL1 |= (UCTR | UCTXSTT | UCTXSTP);
        /* Wait for STOP condition to clear */
        while ((UCBxCTL1 & UCTXSTP));
#if defined(__MSP430_HAS_USCI_AB__)
        if (!(UCxIFG & UCNACKIFG))
            break;
#else
        if (!(UCBxIFG & UCNACKIFG))
            break;
#endif
    }

    /* Wait for the I2C bus to be free */
    while ((UCBxSTAT & UCBBUSY));
    /* Restore IE register */
#if defined(__MSP430_HAS_USCI_AB__)
    UCxIE |= ucbxie_bak;
    /* Allow NACK interrupts */
    UCBxI2CIE |= UCNACKIE;
#else
    UCBxIE = ucbxie_bak;
#endif
    return (timeout)  ?  EEPROM_RESULT_OK  :  EEPROM_RESULT_TIMEOUT;
}

#if defined(__MSP430_HAS_USCI_AB__)

#if (defined(USCIB0_IS_IIC_EEPROM)  &&  defined(UART_0_SUPPORT)) \
    || \
    (defined(USCIB1_IS_IIC_EEPROM)  &&  defined(UART_1_SUPPORT)) \
    || \
    (defined(USCIB2_IS_IIC_EEPROM)  &&  defined(UART_2_SUPPORT)) \
    || \
    (defined(USCIB3_IS_IIC_EEPROM)  &&  defined(UART_3_SUPPORT))
void i2c_eeprom_rx_core(int port)
{
    uint8_t status;

    status = UCBxSTAT;
    /* This ISR is used only if any of the master transmitter's 1 to n-1 bytes
       are Nacked. If the nth byte is Nacked the status flag is modified inside
       the TX ISR itself and NACK ISR is not serviced. */
    if ((status & UCNACKIFG))
    {
        /* Send STOP if slave sends NACK */
        UCBxCTL1 |= UCTXSTP;
        UCBxSTAT &= ~UCNACKIFG;
        result = EEPROM_RESULT_NACK;
    }
    else if ((status & UCSTPIFG))
    {
        UCBxSTAT &= ~UCSTPIFG;
        result = EEPROM_RESULT_OK;
    }
}
#else
    #if defined(USCIB0_IS_IIC_EEPROM)
ISR(USCIAB0RX, USCIAB0RX_isr)
    #elif defined(USCIB1_IS_IIC_EEPROM)
ISR(USCIAB1RX, USCIAB1RX_isr)
    #elif defined(USCIB2_IS_IIC_EEPROM)
ISR(USCIAB2RX, USCIAB2RX_isr)
    #elif defined(USCIB3_IS_IIC_EEPROM)
ISR(USCIAB3RX, USCIAB3RX_isr)
    #endif
{
    uint8_t status;

    status = UCBxSTAT;
    /* This ISR is used only if any of the master transmitter's 1 to n-1 bytes
       are Nacked. If the nth byte is Nacked the status flag is modified inside
       the TX ISR itself and NACK ISR is not serviced. */
    if ((status & UCNACKIFG))
    {
        /* Send STOP if slave sends NACK */
        UCBxCTL1 |= UCTXSTP;
        UCBxSTAT &= ~UCNACKIFG;
        result = EEPROM_RESULT_NACK;
    }
    else if ((status & UCSTPIFG))
    {
        UCBxSTAT &= ~UCSTPIFG;
        result = EEPROM_RESULT_OK;
    }
}
#endif

#if defined(USCIB0_IS_IIC_EEPROM)
ISR(USCIAB0TX, USCIAB0TX_isr)
#elif defined(USCIB1_IS_IIC_EEPROM)
ISR(USCIAB1TX, USCIAB1TX_isr)
#elif defined(USCIB2_IS_IIC_EEPROM)
ISR(USCIAB2TX, USCIAB2TX_isr)
#elif defined(USCIB3_IS_IIC_EEPROM)
ISR(USCIAB3TX, USCIAB3TX_isr)
#endif
{
    if ((UCxIFG & UCBxRXIFG))
    {
        rx_buf[rx_count] = UCBxRXBUF;
        if (++rx_count >= rx_len)
        {
            /* This is the last byte, so raise the stop condition */
            UCBxCTL1 |= UCTXSTP;
        }
    }
    else if ((UCxIFG & UCBxTXIFG))
    {
        if (tx_count < EEPROM_ADDRESS_BYTES)
            UCBxTXBUF = addr_buf[tx_count];
        else
            UCBxTXBUF = tx_buf[tx_count];
        if (++tx_count >= tx_len)
        {
            /* This is the last byte, so raise the stop condition */
            UCBxCTL1 |= UCTXSTP;
        }
    }
}

#elif defined(__MSP430_HAS_EUSCI__)

#if defined(USCIB0_IS_IIC_EEPROM)
ISR(USCI_B0, USCI_B0_isr)
#elif defined(USCIB1_IS_IIC_EEPROM)
ISR(USCI_B1, USCI_B1_isr)
#endif
{
    switch(__even_in_range(UCBxIV, USCI_I2C_UCBIT9IFG))
    {
    case USCI_NONE:
        break;
    case USCI_I2C_UCALIFG:
        result = EEPROM_RESULT_AL;
        break;
    case USCI_I2C_UCNACKIFG:
        result = EEPROM_RESULT_NACK;
        break;
    case USCI_I2C_UCSTTIFG:
        break;
    case USCI_I2C_UCSTPIFG:
        /* Disable TX ready interrupt */
        if (rx_len != 0  &&  rx_count == 0)
        {
            /* Move to the read stage */
            i2c_read_init();
            UCBxTBCNT = rx_len;
            /* I2C TX, start condition */
            UCBxCTL1 |= UCTXSTT;
        }
        else
        {
            result = EEPROM_RESULT_OK;
        }
        break;
    case USCI_I2C_UCRXIFG3:
        break;
    case USCI_I2C_UCTXIFG3:
        break;
    case USCI_I2C_UCRXIFG2:
        break;
    case USCI_I2C_UCTXIFG2:
        break;
    case USCI_I2C_UCRXIFG1:
        break;
    case USCI_I2C_UCTXIFG1:
        break;
    case USCI_I2C_UCRXIFG0:
        rx_buf[rx_count++] = UCBxRXBUF;
        break;
    case USCI_I2C_UCTXIFG0:
        if (tx_count < EEPROM_ADDRESS_BYTES)
            UCBxTXBUF = addr_buf[tx_count];
        else
            UCBxTXBUF = tx_buf[tx_count];
        tx_count++;
        break;
    case USCI_I2C_UCBCNTIFG:
        UCBxIFG &= ~UCTXIFG;
        break;
    case USCI_I2C_UCCLTOIFG:
        result = EEPROM_RESULT_CLOCK_STUCK;
        break;
    case USCI_I2C_UCBIT9IFG:
        /* 9th bit */
        break;
    default:
        break;
    }
}

#else

#if defined(USCIB0_IS_IIC_EEPROM)
ISR(USCI_B0, USCI_B0_isr)
#elif defined(USCIB1_IS_IIC_EEPROM)
ISR(USCI_B1, USCI_B1_isr)
#endif
{
    switch(__even_in_range(UCBxIV, USCI_I2C_UCTXIFG))
    {
    case USCI_NONE:
        break;
    case USCI_I2C_UCALIFG:
        result = EEPROM_RESULT_AL;
        break;
    case USCI_I2C_UCNACKIFG:
        result = EEPROM_RESULT_NACK;
        break;
    case USCI_I2C_UCSTTIFG:
        break;
    case USCI_I2C_UCSTPIFG:
        if (rx_len != 0  &&  rx_count == 0)
        {
            /* Move to the read stage */
            i2c_read_init();
            /* I2C TX, start condition */
            UCBxCTL1 |= UCTXSTT;
        }
        else
        {
            result = EEPROM_RESULT_OK;
        }
        break;
    case USCI_I2C_UCRXIFG:
        rx_buf[rx_count] = UCBxRXBUF;
        if (++rx_count >= rx_len)
        {
            /* This is the last byte, so raise the stop condition */
            UCBxCTL1 |= UCTXSTP;
        }
        break;
    case USCI_I2C_UCTXIFG:
        if (tx_count < EEPROM_ADDRESS_BYTES)
            UCBxTXBUF = addr_buf[tx_count];
        else
            UCBxTXBUF = tx_buf[tx_count];
        if (++tx_count >= tx_len)
        {
            /* This is the last byte, so raise the stop condition */
            UCBxCTL1 |= UCTXSTP;
        }
        break;
    default:
        break;
    }
}
#endif
