/*******************************************************************************
 *  emeter-metrology.h -
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

/*! \file */

#if !defined(_EMETER_METROLOGY_H_)
#define _EMETER_METROLOGY_H_

/* The header files for some devices with sigma-delta converters define both a generic symbol for the type of converter
   they contain and another symbol for the specific quantity of converters. Some only define a symbol for the specific
   number of converters. Here will fill in the symbols for the converter types. */
#if !defined(__MSP430_HAS_SD16__)  && \
           (defined(__MSP430_HAS_SD16_1__) \
        ||  defined(__MSP430_HAS_SD16_2__) \
        ||  defined(__MSP430_HAS_SD16_3__))
#define __MSP430_HAS_SD16__
#endif

#if !defined(__MSP430_HAS_SD16_A__)  && \
           (defined(__MSP430_HAS_SD16_A1__) \
        ||  defined(__MSP430_HAS_SD16_A2__) \
        ||  defined(__MSP430_HAS_SD16_A3__) \
        ||  defined(__MSP430_HAS_SD16_A4__) \
        ||  defined(__MSP430_HAS_SD16_A5__) \
        ||  defined(__MSP430_HAS_SD16_A6__) \
        ||  defined(__MSP430_HAS_SD16_A7__))
#define __MSP430_HAS_SD16_A__
#endif

#if !defined(__MSP430_HAS_SD24__)  && \
           (defined(__MSP430_HAS_SD24_1__) \
        ||  defined(__MSP430_HAS_SD24_2__) \
        ||  defined(__MSP430_HAS_SD24_3__) \
        ||  defined(__MSP430_HAS_SD24_4__))
#define __MSP430_HAS_SD24__
#endif

#if !defined(__MSP430_HAS_SD24_A__)  && \
           (defined(__MSP430_HAS_SD24_A1__) \
        ||  defined(__MSP430_HAS_SD24_A2__) \
        ||  defined(__MSP430_HAS_SD24_A3__) \
        ||  defined(__MSP430_HAS_SD24_A4__))
#define __MSP430_HAS_SD24_A__
#endif

/* Define a generic symbol which indicates the prescence of sigma-delta converters. */
#if     defined(__MSP430_HAS_SD16__) \
    ||  defined(__MSP430_HAS_SD16_A__) \
    ||  defined(__MSP430_HAS_SD24__) \
    ||  defined(__MSP430_HAS_SD24_A__) \
    ||  defined(__MSP430_HAS_SD24_B__)
#define __HAS_SD_ADC__
#endif

#if defined(__HAS_SD_ADC__)
#define SAMPLE_RATE                         4096
#define SAMPLES_PER_10_SECONDS              40960
#endif

#include "metrology-template.h"
#include "metrology-calibration-template.h"

#include "metrology-types.h"

#include "metrology-calibration.h"
#include "metrology-readings.h"

typedef void (*application_init_func_t)(void);
typedef void (*application_poll_func_t)(void);
typedef int (*energy_update_func_t)(int phx, int type, energy_t amount);
typedef void (*energy_pulse_start_func_t)(int ph, int which);
typedef void (*energy_pulse_end_func_t)(int ph, int which);
typedef void (*sag_swell_event_func_t)(int ph, int event);
typedef void (*tamper_event_func_t)(int which, int start);

typedef int (*metrology_init_func_t)(void);
typedef void (*calculate_phase_readings_func_t)(int phx);
typedef void (*calculate_neutral_readings_func_t)(void);
typedef void (*metrology_switch_to_normal_mode_func_t)(void);
typedef void (*metrology_disable_analog_front_end_func_t)(void);
typedef int (*metrology_init_from_nv_data_func_t)(void);

#if !defined(SUPPORT_ABSOLUTE_METROLOGY_LOCATION)

int metrology_init(void);

void calculate_phase_readings(int ph);

#if NUM_PHASES > 1  &&  defined(NEUTRAL_MONITOR_SUPPORT)
void calculate_neutral_readings(void);
#endif

void metrology_switch_to_normal_mode(void);

void metrology_disable_analog_front_end(void);

int metrology_init_from_nv_data(void);

#endif

#include "metrology-callbacks.h"

#if defined(LIMP_MODE_SUPPORT)
void metrology_limp_normal_detection(void);
#endif

#endif
