/*******************************************************************************
 *  metrology-calibration.h -
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

/*! \file */

#if !defined(_METROLOGY_CALIBRATION_H_)
#define _METROLOGY_CALIBRATION_H_

#if defined(__cplusplus)
extern "C"
{
#endif

enum scaling_types_e
{
    SCALING_TYPE_VRMS = 0,
    SCALING_TYPE_LIMP_VRMS,
    SCALING_TYPE_IRMS,
    SCALING_TYPE_LIMP_IRMS,
    SCALING_TYPE_POWER
};

typedef int (*clear_calibration_data_func_t)(void);
typedef calibration_scaling_factor_t (*get_calibration_parameter_func_t)(int phx);
typedef void (*set_calibration_parameter_func_t)(int phx, calibration_scaling_factor_t value);
typedef int16_t (*get_calibration_parameter_func2_t)(int phx);
typedef void (*set_calibration_parameter_func2_t)(int phx, int16_t value);
typedef calibration_scaling_factor_t (*get_calibration_parameter_func3_t)(int phx, int which);
typedef void (*set_calibration_parameter_func3_t)(int phx, int which, calibration_scaling_factor_t value);
typedef int16_t (*get_calibration_parameter_func4_t)(int phx, int which);
typedef void (*set_calibration_parameter_func4_t)(int phx, int which, int16_t value);
typedef int32_t (*get_calibration_parameter_func5_t)(int phx);
typedef void (*set_calibration_parameter_func5_t)(int phx, int32_t value);
typedef int (*get_calibration_status_func_t)(void);
typedef void (*set_calibration_status_func_t)(int value);
typedef int16_t (*get_temperature_intercept_func_t)(void);
typedef int16_t (*get_temperature_slope_func_t)(void);
typedef void (*set_temperature_parameters_func_t)(int16_t temperature_at_calibration, int16_t temperature_sensor_intercept, int16_t temperature_sensor_slope);
typedef int (*align_metrology_with_calibration_data_func_t)(void);

#if !defined(SUPPORT_ABSOLUTE_METROLOGY_LOCATION)

int clear_calibration_data(void);

/*! \brief Get the phase correction calibration factor for a specified phase.
    \param phx The phase number.
    \return The calibration factor.
*/
int16_t get_phase_corr(int phx);

/*! \brief Set the phase correction calibration factor for a specified phase.
    \param phx The phase number.
    \param value The calibration factor.
*/
void set_phase_corr(int phx, int16_t value);

#if defined(DYNAMIC_CURRENT_RELATED_CORRECTION_SUPPORT)  ||  defined(DYNAMIC_FREQUENCY_RELATED_CORRECTION_SUPPORT)
/*! \brief Get the frequency related phase correction calibration factor for a specified phase.
    \param phx The phase number.
    \return The calibration factor.
*/
int16_t get_frequency_related_phase_corr(int phx);

/*! \brief Set the frequency related phase correction calibration factor for a specified phase.
    \param phx The phase number.
    \param The calibration factor.
*/
void set_frequency_related_phase_corr(int phx, int16_t value);

/*! \brief Get the frequency related gain correction calibration factor for a specified phase.
    \param phx The phase number.
    \return The calibration factor.
*/
int16_t get_frequency_related_gain_corr(int phx);

/*! \brief Set the frequency related gain correction calibration factor for a specified phase.
    \param phx The phase number.
    \param The calibration factor.
*/
void set_frequency_related_gain_corr(int phx, int16_t value);
#endif

void set_scaling_factor(int phx, enum scaling_types_e type, calibration_scaling_factor_t parm);
calibration_scaling_factor_t get_scaling_factor(int phx, enum scaling_types_e type);

#if defined(VRMS_SUPPORT)
int16_t get_v_dc_estimate(int phx, int which);

void set_v_dc_estimate(int phx, int which, int16_t value);

int32_t get_v_ac_offset(int phx);

void set_v_ac_offset(int phx, int32_t value);
#endif

#if defined(IRMS_SUPPORT)
int16_t get_i_dc_estimate(int phx, int which);

void set_i_dc_estimate(int phx, int which, int16_t value);

int32_t get_i_ac_offset(int phx);

void set_i_ac_offset(int phx, int32_t value);
#endif

int get_calibration_status(void);

void set_calibration_status(int value);

int16_t get_temperature_intercept(void);

int16_t get_temperature_slope(void);

void set_temperature_parameters(int16_t temperature_at_calibration, int16_t temperature_sensor_intercept, int16_t temperature_sensor_slope);

int align_metrology_with_calibration_data(void);

#endif

#if defined(__cplusplus)
}
#endif

#endif
