/*******************************************************************************
 *  dc_filter24.c - DC estimation and removal for 24 bit signals
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

#include <inttypes.h>
#if defined(__MSP430__)
#include <msp430.h>
#endif
#include "emeter-toolkit.h"

int32_t dc_filter24(int16_t p[3], int32_t x)
{
#if defined(__GNUC__)  &&  defined(__MSP430__)
    int16_t x_ext;
    int32_t y;

    __asm__ (
        " mov.w   %A[x],%A[y] \n"
        " mov.w   %B[x],%B[y] \n"
        " sub.w   2(%[p]),%A[x] \n"
        " subc.w  4(%[p]),%B[x] \n"
        /* Shift, so our final effective shift is 14 bits. */
        " rla.w   %A[x] \n"
        " rlc.w   %B[x] \n"
        " subc.w  %[x_ext],%[x_ext] \n"
        " inv.w   %[x_ext] \n"
        " rla.w   %A[x] \n"
        " rlc.w   %B[x] \n"
        " rlc.w   %[x_ext] \n"

        " add.w   %A[x],0(%[p]) \n"
        " addc.w  2(%[p]),%B[x] \n"
        " addc.w  4(%[p]),%[x_ext] \n"
        /* Store the updated *p */
        " mov.w   %B[x],2(%[p]) \n"
        " mov.w   %[x_ext],4(%[p]) \n"

        " sub.w   %B[x],%A[y] \n"
        " subc.w  %[x_ext],%B[y] \n"
        : [y] "=r"(y)
        : [p] "r"(p), [x] "r"(x), [x_ext] "r"(x_ext));
    return y;
#else
    int64_t tmp;

    tmp = p[2];
    tmp = tmp << 16;
    tmp |= p[1];
    tmp = tmp << 16;
    tmp |= p[0];
    tmp += ((((int32_t) x << 16) - tmp) >> 14);
    x -= (tmp >> 16);
    p[2] = (tmp >> 32) & 0xFFFF;
    p[1] = (tmp >> 16) & 0xFFFF;
    p[0] = tmp & 0xFFFF;
    return x;
#endif
}

int32_t dc_filter24_no_update(const int16_t p[3], int32_t x)
{
#if defined(__GNUC__)  &&  defined(__MSP430__)
    __asm__ (
        " sub.w   2(%[p]),%B[x] \n"
        " subc.w  4(%[p]),%A[x] \n"
        : [x] "+r"(x)
        : [p] "r"(p));
#else
    int64_t tmp;

    tmp = p[2];
    tmp = tmp << 16;
    tmp |= p[1];
    x -= tmp;
    return x;
#endif
}

void dc_filter24_init(int16_t p[3], int16_t x)
{
    /* Put the 16 bit initial estimate where the high 16 bits of the
       24 bit signal will sit when the filter is running */
#if defined(__GNUC__)  &&  defined(__MSP430__)
    __asm__ (
        " mov.w   #0,0(%[p]) \n"
        " mov.b   #0,2(%[p]) \n"
        " mov.b   %[x],3(%[p]) \n"
        " swpb    %[x] \n"
        " sxt     %[x] \n"
        " mov.w   %[x],4(%[p]) \n"
        : 
        : [x] "r"(x), [p] "r"(p));
#else
    int64_t tmp;

    tmp = x;
    tmp = tmp << 16;
    p[2] = (tmp >> 32) & 0xFFFF;
    p[1] = (tmp >> 16) & 0xFFFF;
    p[0] = tmp & 0xFFFF;
#endif
}

int32_t dc_filter24_estimate(const int16_t p[3])
{
#if defined(__GNUC__)  &&  defined(__MSP430__)
    int32_t x;
    int16_t tmp;

    __asm__ (
        " mov.b   4(%[p]),%A[x] \n"
        " swpb    %A[x] \n"
        " mov.b   3(%[p]),%B[x] \n"
        " bis.w   %B[x],%A[x] \n"
        " mov.b   2(%[p]),%B[x] \n"
        " swpb    %B[x] \n"
        " mov.b   1(%[p]),%[tmp] \n"
        " bis.w   %[tmp],%B[x] \n"
        : [x] "=r"(x)
        : [p] "r"(p), [tmp] "r"(tmp));
    return x;
#else
    int64_t tmp;

    tmp = p[2];
    tmp = tmp << 16;
    tmp |= p[1];
    tmp= tmp << 16;
    tmp |= p[0];
    return tmp >> 8;
#endif
}
