/*******************************************************************************
 *  isqrt32.c - Square root of a 32 bit number.
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

#include <inttypes.h>
#include "emeter-toolkit.h"

uint32_t isqrt32(uint32_t h)
{
    uint32_t x;
    uint32_t y;
    int i;

#if defined(__GNUC__)  &&  defined(__MSP430__)
    x = 0;
    y = 0;
    i = 16;
    __asm__ (
        "1: \n"
        " setc \n"
        " rlc.w   %A[x] \n"
        " rlc.w   %B[x] \n"
        " sub.w   %A[x],%A[y] \n"
        " subc.w  %B[x],%B[y] \n"
        " jhs     2f \n"
        " add.w   %A[x],%A[y] \n"
        " addc.w  %B[x],%B[y] \n"
        " sub.w   #2,%A[x] \n"
        "2: \n"
        " inc.w   %A[x] \n"
        " rla.w   %A[h] \n"
        " rlc.w   %B[h] \n"
        " rlc.w   %A[y] \n"
        " rlc.w   %B[y] \n"
        " rla.w   %A[h] \n"
        " rlc.w   %B[h] \n"
        " rlc.w   %A[y] \n"
        " rlc.w   %B[y] \n"
        " dec.w   %[i] \n"
        " jne     1b \n"
        " mov.w   #16,%[i] \n"
        "1: \n"
        " setc \n"
        " rlc.w   %A[x] \n"
        " rlc.w   %B[x] \n"
        " sub.w   %A[x],%A[y] \n"
        " subc.w  %B[x],%B[y] \n"
        " jhs     2f \n"
        " add.w   %A[x],%A[y] \n"
        " addc.w  %B[x],%B[y] \n"
        " sub.w   #2,%A[x] \n"
        "2: \n"
        " inc.w   %A[x] \n"
        " rla.w   %A[y] \n"
        " rlc.w   %B[y] \n"
        " rla.w   %A[y] \n"
        " rlc.w   %B[y] \n"
        " dec.w   %[i] \n"
        " jne     1b \n"
        : [x] "+r"(x), [y] "+r"(y)
        : [h] "r"(h), [i] "r"(i));
#else
    x =
    y = 0;
    for (i = 0;  i < 16;  i++)
    {
        x = (x << 1) | 1;
        if (y < x)
            x -= 2;
        else
            y -= x;
        x++;
        y <<= 1;
        if ((h & 0x80000000))
            y |= 1;
        h <<= 1;
        y <<= 1;
        if ((h & 0x80000000))
            y |= 1;
        h <<= 1;
    }
    for (i = 0;  i < 16;  i++)
    {
        x = (x << 1) | 1;
        if (y < x)
            x -= 2;
        else
            y -= x;
        x++;
        y <<= 2;
    }
#endif
    return  x;
}
