/*******************************************************************************
 *  isqrt64.c - Square root of a 64 bit number.
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

#include <inttypes.h>
#include "emeter-toolkit.h"

#if defined(EMETER_TOOLKIT_SUPPORT_64BIT)
uint64_t isqrt64(uint64_t h)
{
    uint64_t x;
    uint64_t y;
    int i;

    /* The answer is calculated as a 64 bit value, where the last
       32 bits are fractional.
       Calling with negative numbers is not a good idea :-) */
#if defined(__GNUC__)  &&  defined(__MSP430__)
    x = 0;
    y = 0;
    __asm__ (
        /* Use a variable on the stack as the loop counter */
        " mov.w  #32,%A[x] \n"
        " push.w %A[x] \n"
        " clr.w  %A[x] \n"
        "1: \n"
        " setc \n"
        " rlc.w   %A[x] \n"
        " rlc.w   %B[x] \n"
        " rlc.w   %C[x] \n"
        " rlc.w   %D[x] \n"
        " sub.w   %A[x],%A[y] \n"
        " subc.w  %B[x],%B[y] \n"
        " subc.w  %C[x],%C[y] \n"
        " subc.w  %D[x],%D[y] \n"
        " jhs     2f \n"
        " add.w   %A[x],%A[y] \n"
        " addc.w  %B[x],%B[y] \n"
        " addc.w  %C[x],%C[y] \n"
        " addc.w  %D[x],%D[y] \n"
        " sub.w   #2,%A[x] \n"
        "2: \n"
        " inc.w   %A[x] \n"
        " rla.w   %A[h] \n"
        " rlc.w   %B[h] \n"
        " rlc.w   %C[h] \n"
        " rlc.w   %D[h] \n"
        " rlc.w   %A[y] \n"
        " rlc.w   %B[y] \n"
        " rlc.w   %C[y] \n"
        " rlc.w   %D[y] \n"
        " rla.w   %A[h] \n"
        " rlc.w   %B[h] \n"
        " rlc.w   %C[h] \n"
        " rlc.w   %D[h] \n"
        " rlc.w   %A[y] \n"
        " rlc.w   %B[y] \n"
        " rlc.w   %C[y] \n"
        " rlc.w   %D[y] \n"
        " dec.w   0(sp) \n"
        " jne     1b \n"
        " mov.w   #32,0(sp) \n"
        "1: \n"
        " setc \n"
        " rlc.w   %A[x] \n"
        " rlc.w   %B[x] \n"
        " rlc.w   %C[x] \n"
        " rlc.w   %D[x] \n"
        " sub.w   %A[x],%A[y] \n"
        " subc.w  %B[x],%B[y] \n"
        " subc.w  %C[x],%C[y] \n"
        " subc.w  %D[x],%D[y] \n"
        " jhs     2f \n"
        " add.w   %A[x],%A[y] \n"
        " addc.w  %B[x],%B[y] \n"
        " addc.w  %C[x],%C[y] \n"
        " addc.w  %D[x],%D[y] \n"
        " sub.w   #2,%A[x] \n"
        "2: \n"
        " inc.w   %A[x] \n"
        " rla.w   %A[y] \n"
        " rlc.w   %B[y] \n"
        " rlc.w   %C[y] \n"
        " rlc.w   %D[y] \n"
        " rla.w   %A[y] \n"
        " rlc.w   %B[y] \n"
        " rlc.w   %C[y] \n"
        " rlc.w   %D[y] \n"
        " dec.w   0(sp) \n"
        " jne     1b \n"
        " pop.w   %A[x] \n"
        : [x] "+r"(x), [y] "+r"(y)
        : [h] "r"(h));
#else
    x =
    y = 0;
    for (i = 0;  i < 32;  i++)
    {
        x = (x << 1) | 1;
        if (y < x)
            x -= 2;
        else
            y -= x;
        x++;
        y <<= 1;
        if ((h & 0x8000000000000000ULL))
            y |= 1;
        h <<= 1;
        y <<= 1;
        if ((h & 0x8000000000000000ULL))
            y |= 1;
        h <<= 1;
    }
    for (i = 0;  i < 32;  i++)
    {
        x = (x << 1) | 1;
        if (y < x)
            x -= 2;
        else
            y -= x;
        x++;
        y <<= 2;
    }
#endif
    return  x;
}
#endif
