/*******************************************************************************
 *  q1_15_mulq.c - Multiply two 16 bit numbers
 *
 * Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/ 
 * ALL RIGHTS RESERVED 
 *
 ******************************************************************************/

#include <inttypes.h>
#if defined(__MSP430__)
#include <msp430.h>
#endif
#include "emeter-toolkit.h"

uint16_t rndnum;
uint16_t tpdnum;

int16_t Q1_15_mulq(register int16_t x, register int16_t y)
{
    int16_t x1;
    int32_t z;

#if defined(__GNUC__)  &&  defined(__MSP430__)
#if (defined(__MSP430_HAS_MPY__)  ||  defined(__MSP430_HAS_MPY32__))  &&  !defined(__TOOLKIT_USE_SOFT_MPY__)
    __asm__ (
"__MPY=0x130 \n"
"__MPYS=0x132 \n"
"__MAC=0x134 \n"
"__MACS=0x136 \n"
"__OP2=0x138 \n"
"__RESLO=0x13a \n"
"__RESHI=0x13c \n"
"__SUMEXT=0x13e \n"
        " push.w  r2 \n"
        " dint \n"
        " mov   %[x],&__MPYS \n"
        " mov   %[y],&__OP2 \n"
        " mov   &__RESHI,%B[z] \n"
        " mov   &__RESLO,%A[z] \n"
        " pop.w r2 \n"
        //Do TPD dithering at the Q1.15 level
        " add   %4,%A[z] \n"
        " addc  #0,%B[z] \n"
        //Shift to Q1.15 format (i.e. the top 16 bits are returned)
        " rla   %A[z] \n"
        " rlc   %B[z] \n"
        " mov   %B[z],%[x1] \n"
        : [z] "=r"(z), [x1] "=r"(x1)
        : [x] "r"(x), [y] "r"(y), [tpdnum] "r"(tpdnum));
#else
    z = 0;
    x1 = 0;
    __asm__ (
        " tst   %[x] \n"
        " jge   2f \n"
        " mov   #-1,%[x1] \n"
        " jmp   2f \n"
        "6: \n"
        " add   %[x],%A[z] \n"
        " addc  %[x1],%B[z] \n"
        "1: \n"
        " rla   %[x] \n"
        " rlc   %[x1] \n"
        "2: \n"
        " rra   %[y] \n"
        " jc    5f \n"
        " jne   1b \n"
        " jmp   4f \n"
        "5: \n"
        " sub   %[x],%A[z] \n"
        " subc  %[x1],%B[z] \n"
        "3: \n"
        " rla   %[x] \n"
        " rlc   %[x1] \n"
        " rra   %[y] \n"
        " jnc   6b \n"
        " cmp   #0xFFFF,%[y] \n"
        " jne   3b \n"
        "4: \n"
        //Do TPD dithering at the Q1.15 level
        " add   %[tpdnum],%A[z] \n"
        " addc  #0,%B[z] \n"
        //Shift to Q1.15 format (i.e. the top 16 bits are returned)
        " rla   %A[z] \n"
        " rlc   %B[z] \n"
        " mov   %B[z],%[x1] \n"
        : [z] "+r"(z), [x1] "+r"(x1)
        : [x] "r"(x), [y] "r"(y), [tpdnum] "r"(tpdnum));
#endif
#else
    z = ((int32_t) x*y) + tpdnum;
    x1 = (z >> 15);
#endif
    return x1;
}

void tpd_gen(void)
{
    int16_t x1;
    int16_t x2;

    x1 = rndnum;
#if defined(__GNUC__)  &&  defined(__MSP430__)
    x2 = 0;
    __asm__ (
        " sub   %[x1],%[x2] \n"     //0
        " rla   %[x1] \n"
        " add   %[x1],%[x2] \n"     //1     
        " rla   %[x1] \n"
        " sub   %[x1],%[x2] \n"     //2     
        " rla   %[x1] \n"
        " rla   %[x1] \n"
        " add   %[x1],%[x2] \n"     //4
        " rla   %[x1] \n"
        " rla   %[x1] \n"
        " sub   %[x1],%[x2] \n"     //6     
        " rla   %[x1] \n"
        " add   %[x1],%[x2] \n"     //7     
        " rla   %[x1] \n"
        " rla   %[x1] \n"
        " rla   %[x1] \n"
        " sub   %[x1],%[x2] \n"     //10
        : [x2] "+r"(x2), [x1] "+r"(x1));
#else
    x2 = x1*31821U;
#endif

    x2 = (x2 + 13849) & 0x3FFF;
    tpdnum = rndnum + x2;
    rndnum = x2;
}
